//////////////////////////////////////////////////////////////////////////
//
// Nano River Technologies
//
// File:         MiniBoard Class Library C++ File (miniboard_class.cpp)
//
// Desciption:   This is the C++ file for the MiniBoard Class Library
//
// Revision:     Version 1.0
//
//////////////////////////////////////////////////////////////////////////

#include "Usb.h"
#include "Miniboard.h"
#include "MiniboardClass.h"
#include <windows.h>

#define VENDORID    0x2058
#define PRODUCTID   0x1004


char     buf[256];
int      j;
LONG     bytesToXfer;
struct   usb_device *usb_dev;
struct   usb_dev_handle *usb_handle;
int      ret;



MBc::MBc(void)
{
}

MBc::~MBc(void)
{
}


//////////////////////////////////////
// Misc
//////////////////////////////////////
/* 
 *  Search of our USB device on the BUS
 */
struct usb_device *device_init( unsigned int vendorid, unsigned int productid )
{
    struct usb_bus *usb_bus;
    struct usb_device *dev;

    /* Generic USB library bus structure traversal */
    usb_find_busses( );
    usb_find_devices( );

    /* Scan device tree for vendor ID/product ID match */
    for ( usb_bus = usb_get_busses( ); usb_bus; usb_bus = usb_bus -> next )
    {
        for ( dev = usb_bus -> devices; dev; dev = dev -> next )
        {
            if (( dev->descriptor.idVendor == vendorid ) &&
                ( dev->descriptor.idProduct == productid ) )
                    return dev;
        }
    }

    return NULL;
}





///////////////////////////////////////
// GPIO Tasks
///////////////////////////////////////

NANO_RESULT	MBc::Nano_GPIOSetDirection ( 
		HANDLE 	hDevice, 
		ULONG 	Value, 
		ULONG 	Mask
	)
{
	ret=-1;
	bytesToXfer = 9;
	buf[0] = 0x00;
	buf[1] = ((Value & 0xFF000000) >> 24);
	buf[2] = ((Value & 0x00FF0000) >> 16);
	buf[3] = ((Value & 0x0000FF00) >> 8);
	buf[4] = ((Value & 0x000000FF));
	buf[5] = ((Mask & 0xFF000000) >> 24);
	buf[6] = ((Mask & 0x00FF0000) >> 16);
	buf[7] = ((Mask & 0x0000FF00) >> 8);
	buf[8] = ((Mask & 0x000000FF));
	ret = usb_control_msg(usb_handle, 0x40, 0xdd, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}
	
NANO_RESULT	MBc::Nano_GPIOGetDirection (
		HANDLE 	hDevice, 
		ULONG  	*pValue 
	)
{
	ret=-1;
	bytesToXfer = 9;
    ret = usb_control_msg(usb_handle, 0xC0, 0xdd, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	*pValue = ((unsigned char(buf[5])<<24) + (unsigned char(buf[6])<<16) + (unsigned char(buf[7])<<8) + unsigned char(buf[8]));
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_GPIOWrite (
		HANDLE 	hDevice, 
		ULONG 	Value, 
		ULONG 	Mask
	)
{
	ret=-1;
	bytesToXfer = 9;
	buf[0] = 0x01;
	buf[1] = ((Value & 0xFF000000) >> 24);
	buf[2] = ((Value & 0x00FF0000) >> 16);
	buf[3] = ((Value & 0x0000FF00) >> 8);
	buf[4] = ((Value & 0x000000FF));
	buf[5] = ((Mask & 0xFF000000) >> 24);
	buf[6] = ((Mask & 0x00FF0000) >> 16);
	buf[7] = ((Mask & 0x0000FF00) >> 8);
	buf[8] = ((Mask & 0x000000FF));
	ret = usb_control_msg(usb_handle, 0x40, 0xdd, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}
	
NANO_RESULT	MBc::Nano_GPIORead (
		HANDLE 	hDevice, 
		ULONG 	*pValue
	)
{
	ret=-1;
	bytesToXfer = 9;
    ret = usb_control_msg(usb_handle, 0xC0, 0xdd, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	*pValue = ((unsigned char(buf[1])<<24) + (unsigned char(buf[2])<<16) + (unsigned char(buf[3])<<8) + unsigned char(buf[4]));
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}
	
NANO_RESULT	MBc::Nano_GPIOSetSingleBitDirection (
		HANDLE 	hDevice, 
		ULONG 	GPIONumber, 
		BOOL 	bOutput
	)
{
	ret=-1;
	bytesToXfer = 3;
	buf[0] = 0x02;
	buf[1] = GPIONumber & 0x3F;
	if (bOutput) {buf[2]=0x01;} else {buf[2]=0x00;}
	ret = usb_control_msg(usb_handle, 0x40, 0xdd, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}
	
NANO_RESULT	MBc::Nano_GPIOGetSingleBitDirection (
		HANDLE 	hDevice, 
		ULONG 	GPIONumber, 
		BOOL 	*pbOutput
	)
{
	ret=-1;
	unsigned char val;
	unsigned char msk;
	bytesToXfer = 9;
    ret = usb_control_msg(usb_handle, 0xC0, 0xdd, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (GPIONumber <= 7) {
		msk = 0x01 << (GPIONumber);
		val = buf[8] & msk;
		if (val==0x0) {*pbOutput=false;} else {*pbOutput=true;}
	} else if (GPIONumber <= 15) {
		msk = 0x01 << (GPIONumber-8);
		val = buf[7] & msk;
		if (val==0x0) {*pbOutput=false;} else {*pbOutput=true;}
	} else if (GPIONumber <= 23) {
		msk = 0x01 << (GPIONumber-16);
		val = buf[6] & msk;
		if (val==0x0) {*pbOutput=false;} else {*pbOutput=true;}
	} else if (GPIONumber <= 31) {
		msk = 0x01 << (GPIONumber-24);
		val = buf[5] & msk;
		if (val==0x0) {*pbOutput=false;} else {*pbOutput=true;}
	} 
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_GPIOSingleBitWrite (
		HANDLE 	hDevice, 
		ULONG 	GPIONumber, 
		BOOL 	Value
	)
{
	ret=-1;
	bytesToXfer = 3;
	buf[0] = 0x03;
	buf[1] = GPIONumber & 0x3F;
	if (Value) {buf[2]=0x01;} else {buf[2]=0x00;}
	ret = usb_control_msg(usb_handle, 0x40, 0xdd, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}
	
NANO_RESULT	MBc::Nano_GPIOSingleBitRead (
		HANDLE 	hDevice, 
		ULONG 	GPIONumber, 
		BOOL 	*pValue
		) 
{
	ret=-1;
	unsigned char val;
	unsigned char msk;
	bytesToXfer = 9;
    ret = usb_control_msg(usb_handle, 0xC0, 0xdd, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (GPIONumber <= 7) {
		msk = 0x01 << (GPIONumber);
		val = buf[4] & msk;
		if (val==0x0) {*pValue=false;} else {*pValue=true;}
	} else if (GPIONumber <= 15) {
		msk = 0x01 << (GPIONumber-8);
		val = buf[3] & msk;
		if (val==0x0) {*pValue=false;} else {*pValue=true;}
	} else if (GPIONumber <= 23) {
		msk = 0x01 << (GPIONumber-16);
		val = buf[2] & msk;
		if (val==0x0) {*pValue=false;} else {*pValue=true;}
	} else if (GPIONumber <= 31) {
		msk = 0x01 << (GPIONumber-24);
		val = buf[1] & msk;
		if (val==0x0) {*pValue=false;} else {*pValue=true;}
	} 
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}


///////////////////////////////////////
// IIC Tasks
///////////////////////////////////////

NANO_RESULT	MBc::Nano_IICSetFrequency ( 
		HANDLE 	hDevice, 
		BYTE 	Frequency 
		)
{
	ret=-1;
	bytesToXfer = 2;
	buf[0] = 0x00;
	buf[1] = Frequency;
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_IICGetFrequency ( 
		HANDLE 	hDevice, 
		BYTE 	*pFrequency 
		)
{
	ret=-1;
	bytesToXfer = 11;
	ret = usb_control_msg(usb_handle, 0xC0, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	*pFrequency  = buf[3];
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_IICRead ( 
		HANDLE 	hDevice, 
		BYTE    SlaveAddress,
        BYTE    StartAddressLength,
		ULONG   StartAddress,
		WORD    BufferLength,
		BYTE    *pInBuffer
		)
{
	ret=-1;
	bool error;
	error = false;
	bool t_error;
	t_error = false;
    WORD tmpword;


	//////////////////////////////////////////////////
	// DUMMY Write to get the address correct
    //////////////////////////////////////////////////

	// Write out start address into the first few buffer locations 
	ULONG start_address;
	BYTE start_address_byte_0,start_address_byte_1,start_address_byte_2,start_address_byte_3;
	BYTE a_size;

	a_size = StartAddressLength;
	if (StartAddressLength > 4) {a_size=0;}      // Fix address size to be 0-4
    start_address = StartAddress & 0x000000FF;   // Form the start address as part of the data
    start_address_byte_0 = start_address;
    start_address = StartAddress & 0x0000FF00;
    start_address_byte_1 = start_address >> 8;
    start_address = StartAddress & 0x00FF0000;
    start_address_byte_2 = start_address >> 16;
    start_address = StartAddress & 0xFF000000;
    start_address_byte_3 = start_address >> 24;
	if (a_size == 1) {
		buf[0] = start_address_byte_0;
	} else if (a_size == 2) {
		buf[0] = start_address_byte_1;
		buf[1] = start_address_byte_0;
	} else if (a_size == 3) {
		buf[0] = start_address_byte_2;
		buf[1] = start_address_byte_1;
		buf[2] = start_address_byte_0;
	} else if (a_size == 4) {
		buf[0] = start_address_byte_3;
		buf[1] = start_address_byte_2;
		buf[2] = start_address_byte_1;
		buf[3] = start_address_byte_0;
	}
	bytesToXfer = a_size;
	if (a_size != 0) {
		ret = usb_control_msg(usb_handle, 0x40, 0xe0, 0x0000, 0x0000, buf, bytesToXfer, 1000);
		if (ret!=bytesToXfer) {error=true;}
	}


	// Slave Address
	bytesToXfer = 2;
	buf[0] = 0x03;
	buf[1] = char(SlaveAddress);
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	if (a_size != 0) {

		// Buffer Length
		bytesToXfer = 3;
		buf[0] = 0x04;
		buf[1] = 0x00;
	    buf[2] = char(a_size);       
		ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
		if (ret!=bytesToXfer) {error=true;}

		// Write Command
		bytesToXfer = 2;
		buf[0] = 0x08;
		buf[1] = 0x02;
		ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
		if (ret!=bytesToXfer) {error=true;}

		//check error flag
		bytesToXfer = 3;
		ret = usb_control_msg(usb_handle, 0xC0, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
		if (ret!=bytesToXfer) {error=true;}
		if (buf[2]!=0x0) {t_error=true;}     // IIC Error

	}

	//////////////////////////////////////////////////
	// REAL data read
    //////////////////////////////////////////////////

	// Buffer Length
	bytesToXfer = 3;
	buf[0] = 0x04;
	tmpword = BufferLength;
	buf[2] = tmpword & 0xFF;
    tmpword  = tmpword >> 8;
	buf[1] = tmpword && 0x01;

	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	// Read Command
	bytesToXfer = 2;
	buf[0] = 0x08;
	buf[1] = 0x01;
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

    //check error flag
	bytesToXfer = 3;
	ret = usb_control_msg(usb_handle, 0xC0, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}
	if (buf[2]!=0x0) {t_error=true;}       // IIC Error

	// Transfer the Data
	bytesToXfer = 64;
	ret = usb_control_msg(usb_handle, 0xC0, 0xe0, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}


	if (BufferLength < 64) 
	{
		for (j=0;   j<BufferLength;  j++) {pInBuffer[j] = buf[j];}
	} else 
	{
		for (j=0;   j<64;  j++) {pInBuffer[j] = buf[j];}
		bytesToXfer = 64;
	    ret = usb_control_msg(usb_handle, 0xC0, 0xe1, 0x0000, 0x0000, buf, bytesToXfer, 1000);
		if (ret!=bytesToXfer) {error=true;}
		if (BufferLength < 128)
		{
			for (j=0;   j<BufferLength-64;  j++) {pInBuffer[j+64] = buf[j];}
		} else
		{
			for (j=0;   j<64;  j++) {pInBuffer[j+64] = buf[j];}
			bytesToXfer = 64;
         	ret = usb_control_msg(usb_handle, 0xC0, 0xe2, 0x0000, 0x0000, buf, bytesToXfer, 1000);
			if (ret!=bytesToXfer) {error=true;}
			if (BufferLength < 192)
			{
				for (j=0;   j<BufferLength-128;  j++) {pInBuffer[j+128] = buf[j];}
			} else
			{
				for (j=0;   j<64;  j++) {pInBuffer[j+128] = buf[j];}
				bytesToXfer = 64;
	            ret = usb_control_msg(usb_handle, 0xC0, 0xe3, 0x0000, 0x0000, buf, bytesToXfer, 1000);
				if (ret!=bytesToXfer) {error=true;}
				for (j=0;   j<BufferLength-192;  j++) {pInBuffer[j+192] = buf[j];}
			}
		}
	}

	if (error==true) {return NANO_XACTION_FAILURE;} 
	else if (t_error==true) {return NANO_IIC_PROTOCOL_ERROR;} 
	else {return NANO_SUCCESS;}
}


NANO_RESULT	MBc::Nano_IICWrite ( 
		HANDLE 	hDevice, 
		BYTE    SlaveAddress,
        BYTE    StartAddressLength,
		ULONG   StartAddress,
		WORD    BufferLength,
		BYTE    *pOutBuffer
		)
{
	ret=-1;
	bool error;
	error = false;
	bool t_error;
	t_error = false;
    WORD tmpword;

	// Form the data buffer
	bytesToXfer = 64;

	ULONG start_address;
	BYTE start_address_byte_0,start_address_byte_1,start_address_byte_2,start_address_byte_3;
	BYTE a_size;
	int i;

	a_size = StartAddressLength;
	if (StartAddressLength > 4) {a_size=0;}      // Fix address size to be 0-4
    start_address = StartAddress & 0x000000FF;   // Form the start address as part of the data
    start_address_byte_0 = start_address;
    start_address = StartAddress & 0x0000FF00;
    start_address_byte_1 = start_address >> 8;
    start_address = StartAddress & 0x00FF0000;
    start_address_byte_2 = start_address >> 16;
    start_address = StartAddress & 0xFF000000;
    start_address_byte_3 = start_address >> 24;
	if (a_size == 1) {
		buf[0] = start_address_byte_0;
	} else if (a_size == 2) {
		buf[0] = start_address_byte_1;
		buf[1] = start_address_byte_0;
	} else if (a_size == 3) {
		buf[0] = start_address_byte_2;
		buf[1] = start_address_byte_1;
		buf[2] = start_address_byte_0;
	} else if (a_size == 4) {
		buf[0] = start_address_byte_3;
		buf[1] = start_address_byte_2;
		buf[2] = start_address_byte_1;
		buf[3] = start_address_byte_0;
	}

	for (i=0;i<64-a_size;i++) {
		buf[i+a_size] = pOutBuffer[i];
	}
	ret = usb_control_msg(usb_handle, 0x40, 0xe0, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}
	if (bytesToXfer!=64) {error=true;}

	bytesToXfer = 64;
	for (i=0;i<64;i++) {
		buf[i] = pOutBuffer[i+64-a_size];
	}
	ret = usb_control_msg(usb_handle, 0x40, 0xe1, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	bytesToXfer = 64;
	for (i=0;i<64;i++) {
		buf[i] = pOutBuffer[i+128-a_size];
	}
	ret = usb_control_msg(usb_handle, 0x40, 0xe2, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	bytesToXfer = 64;
	for (i=0;i<64;i++) {
		buf[i] = pOutBuffer[i+192-a_size];
	}
	ret = usb_control_msg(usb_handle, 0x40, 0xe3, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	if (a_size!=0) {
		bytesToXfer = a_size;
		for (i=0;i<a_size;i++) {
			buf[i] = pOutBuffer[i+256-a_size];
		}
		ret = usb_control_msg(usb_handle, 0x40, 0xdf, 0x0000, 0x0000, buf, bytesToXfer, 1000);
		if (ret!=bytesToXfer) {error=true;}
	}

	// Slave Address
	bytesToXfer = 2;
	buf[0] = 0x03;
	buf[1] = char(SlaveAddress);
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	// Buffer Length
	bytesToXfer = 3;
	buf[0] = 0x04;
	tmpword = BufferLength+a_size;
    tmpword = tmpword & 0xFF;
	buf[2] = tmpword;
	tmpword = BufferLength+2;
    tmpword  = tmpword >> 8;
	tmpword  = tmpword && 0x01;
	buf[1] = tmpword;
	
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	// Write Command
	bytesToXfer = 2;
	buf[0] = 0x08;
	buf[1] = 0x02;
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

    //check error flag
	bytesToXfer = 3;
	ret = usb_control_msg(usb_handle, 0xC0, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}
	if (buf[2]!=0x0) {t_error=true;}      // IIC Error

	if (error==true) {return NANO_XACTION_FAILURE;} 
	else if (t_error==true) {return NANO_IIC_PROTOCOL_ERROR;} 
	else {return NANO_SUCCESS;}
}


NANO_RESULT	MBc::Nano_IICScanConnectedDevices ( 
		HANDLE 	hDevice, 
		DEV_LIST *pList
		)
{
	ret=-1;
	bool error;
	char SlaveAddress;
	SlaveAddress = 0x50;
	error = false;
	DEV_LIST lst;

	// Scan loop
	for (j=0; j<128; j++) {

		// Slave Address
		bytesToXfer = 2;
		buf[0] = 0x03;
		buf[1] = char(j);
	    ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
		if (ret!=bytesToXfer) {error=true;}

		// Buffer Length
		bytesToXfer = 3;
		buf[0] = 0x04;
		buf[1] = 0x00;
		buf[2] = 0x00;
	    ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
		if (ret!=bytesToXfer) {error=true;}

		// Write Command
		bytesToXfer = 2;
		buf[0] = 0x08;
		buf[1] = 0x02;
	    ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
		if (ret!=bytesToXfer) {error=true;}

		//check error flag
		bytesToXfer = 3;
	    ret = usb_control_msg(usb_handle, 0xC0, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
		if (ret!=bytesToXfer) {error=true;}

		if (buf[2]!=0x0) {lst.List[j] = false;} else {lst.List[j] = true;}

	}
	*pList = lst;
	if (error==true) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_IICReadSCL ( 
		HANDLE 	hDevice, 
		BOOL 	*pState
		)
{
    ret=-1;
	bool error;
	error = false;


	// Read SCL Command
	bytesToXfer = 2;
	buf[0] = 0x08;
	buf[1] = 0x0C;
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	// Read SCL register
	bytesToXfer = 13;
	ret = usb_control_msg(usb_handle, 0xC0, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}
	if (buf[10] == 0x01) {*pState=true;} else {*pState=false;}
	if (error==true) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_IICReadSDA ( 
		HANDLE 	hDevice, 
		BOOL 	*pState
		)
{
    ret=-1;
	bool error;
	error = false;

	// Read SDA Command
	bytesToXfer = 2;
	buf[0] = 0x08;
	buf[1] = 0x0D;
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	// Read SDA register
	bytesToXfer = 13;
	ret = usb_control_msg(usb_handle, 0xC0, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}
	if (buf[11] == 0x01) {*pState=true;} else {*pState=false;}
	if (error==true) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_IICWriteSCL ( 
		HANDLE 	hDevice, 
		BOOL 	pState
		)
{
    ret=-1;
	bool error;
	error = false;

	// Write SCL register
	bytesToXfer = 2;
	buf[0] = 0x06;
	if (pState) {buf[1] = 0x01;} else {buf[1] = 0x00;};
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	// Write SCL Command
	bytesToXfer = 2;
	buf[0] = 0x08;
	buf[1] = 0x0E;
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	if (error==true) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_IICWriteSDA ( 
		HANDLE 	hDevice, 
		BOOL 	pState
		)
{
    ret=-1;
	bool error;
	error = false;

	// Write SDA register
	bytesToXfer = 2;
	buf[0] = 0x07;
	if (pState) {buf[1] = 0x01;} else {buf[1] = 0x00;};
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	// Write SDA Command
	bytesToXfer = 2;
	buf[0] = 0x08;
	buf[1] = 0x0F;
	ret = usb_control_msg(usb_handle, 0x40, 0xde, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	if (error==true) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}




///////////////////////////////////////
// SPI Tasks
///////////////////////////////////////

NANO_RESULT	MBc::Nano_SPISetClockConfig ( 
		HANDLE 	hDevice, 
		BYTE 	CPOL, 
		BYTE 	CPHA	
		)
{
    ret=-1;
	bytesToXfer = 2;
	buf[0] = 0x00;
	BYTE i;
	i = CPOL & 0x01;
	i = i << 1;
	buf[1] = CPHA & 0x01;
	buf[1] = buf[1] | i;
	ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_SPIGetClockConfig ( 
		HANDLE 	hDevice, 
		BYTE 	*pCPOL, 
		BYTE 	*pCPHA 
		)
{
    ret=-1;
	bytesToXfer = 16;
	ret = usb_control_msg(usb_handle, 0xC0, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	BYTE i;
    i = buf[3] & 0x02;
    *pCPOL = i >> 1;
	*pCPHA = buf[3] & 0x01;
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_SPISetFrequency ( 
		HANDLE 	hDevice, 
		BYTE 	Frequency 
		)
{
    ret=-1;
	bytesToXfer = 2;
	buf[0] = 0x01;
	buf[1] = Frequency;
	ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_SPIGetFrequency ( 
		HANDLE 	hDevice, 
		BYTE 	*pFrequency		
		)
{
    ret=-1;
	bytesToXfer = 16;
	ret = usb_control_msg(usb_handle, 0xC0, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	*pFrequency = buf[4];
	if (ret!=bytesToXfer) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}

NANO_RESULT	MBc::Nano_SPISetChannelConfig ( 
		HANDLE 	hDevice, 
		BYTE 	Channel, 
		BYTE 	Configuration 
		)
{

	ULONG en;
	ULONG hi_lo;
	ULONG mask;
    ret=-1;
	bool error;
	error = false;

	if ((Channel==0) || (Channel>32)) {
		return NANO_BAD_PARAMETER;
	}
	else {


       // Transfer to get the existing configuration
	   bytesToXfer = 14;
	   ret = usb_control_msg(usb_handle, 0xC0, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	   if (ret!=bytesToXfer) {error=true;}
	   en    = ((unsigned char(buf[6])<<24) + (unsigned char(buf[7])<<16) + (unsigned char(buf[8])<<8) + unsigned char(buf[9]));
	   hi_lo    = ((unsigned char(buf[10])<<24) + (unsigned char(buf[11])<<16) + (unsigned char(buf[12])<<8) + unsigned char(buf[13]));

	   // Update the configuration
	   mask = 0x1 << (Channel-1);
	   if (Configuration == 1) {
		   en    = en | mask;
		   hi_lo = hi_lo & ~mask;
	   } else if (Configuration == 2) {
	   	   en    = en | mask;
		   hi_lo = hi_lo | mask;
	   } else {
		   en    = en & ~mask;
		   hi_lo = hi_lo & ~mask;
	   }

	   // Send the update
	   buf[0] = 0x03;
	   buf[1] = ((en & 0xFF000000) >> 24);
	   buf[2] = ((en & 0x00FF0000) >> 16);
	   buf[3] = ((en & 0x0000FF00) >> 8);
	   buf[4] = ((en & 0x000000FF));
	   buf[5] = ((hi_lo & 0xFF000000) >> 24);
	   buf[6] = ((hi_lo & 0x00FF0000) >> 16);
	   buf[7] = ((hi_lo & 0x0000FF00) >> 8);
	   buf[8] = ((hi_lo & 0x000000FF));

	   bytesToXfer = 9;
	   ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	   if (ret!=bytesToXfer) {error=true;}

	   if (error!=true) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
	}
}

NANO_RESULT	MBc::Nano_SPIGetChannelConfig ( 
		HANDLE 	hDevice, 
		BYTE 	Channel, 
		BYTE 	*pConfiguration 
		)
{
	ULONG en;
	ULONG hi_lo;
	ULONG mask;
    ret=-1;
	bool error;
	error = false;

	if ((Channel==0) || (Channel>32)) {
        *pConfiguration = 0;
		return NANO_BAD_PARAMETER;
	}
	else {


       // Transfer to get the existing configuration
	   bytesToXfer = 14;
	   ret = usb_control_msg(usb_handle, 0xC0, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	   if (ret!=bytesToXfer) {error=true;}
	   en    = ((unsigned char(buf[6])<<24) + (unsigned char(buf[7])<<16) + (unsigned char(buf[8])<<8) + unsigned char(buf[9]));
	   hi_lo    = ((unsigned char(buf[10])<<24) + (unsigned char(buf[11])<<16) + (unsigned char(buf[12])<<8) + unsigned char(buf[13]));

	   // Update the configuration
	   mask = 0x1 << (Channel-1);

       en = en & mask;
	   hi_lo = hi_lo & mask;

	   if (en==0){
		   *pConfiguration = 0;
	   } else if (hi_lo==0) {
		   *pConfiguration = 1;
	   } else {
		   *pConfiguration = 2;
	   }

	   if (error==true) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
	}
}



NANO_RESULT	MBc::Nano_SPIReadWrite ( 
		HANDLE 	hDevice, 
		BYTE 	Channel, 
		BYTE 	*pInBuffer, 
		BYTE 	*pOutBuffer, 
		WORD 	Length
		)
{
    ret=-1;
	bool error;
	error = false;
    WORD bytes_left;
	WORD bytes_sent;
	BYTE transfer_size;
	BYTE send_stop;

	// Common Stuff
	int i;

    // Write to the CS channel (allows for GPIO based chip selects)
	bytesToXfer = 2;
	buf[0] = 0x02;
	buf[1] = Channel;
	ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	// Init SPI bus
	bytesToXfer = 2;
	buf[0] = 0x06;
	buf[1] = 0x01;
	ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

    bytes_left = Length;
    bytes_sent = 0;

	while (bytes_left>0) {
		
		if (bytes_left > 128) 
		{
			transfer_size = 128;
			send_stop = 0x0;
			bytes_left = bytes_left - 128;
		} else
		{
			transfer_size = bytes_left;
			send_stop = 0x1;
			bytes_left = 0;
		}

	    // Write to byte length register
	    bytesToXfer = 2;
	    buf[0] = 0x05;
	    buf[1] = transfer_size;
	    ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	    if (ret!=bytesToXfer) {error=true;}

	    // Init Out buffer
		if (transfer_size > 64)
			bytesToXfer = 64;
		else
			bytesToXfer = transfer_size;
	    for (i=0; i<bytesToXfer; i++) {
		    buf[i] = pOutBuffer[i+bytes_sent];
	    }
	    ret = usb_control_msg(usb_handle, 0x40, 0xe8, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	    if (ret!=bytesToXfer) {error=true;}

		if (transfer_size > 64) {
			bytesToXfer = transfer_size - 64;
			for (i=0; i<bytesToXfer; i++) {
				buf[i] = pOutBuffer[i+64 + bytes_sent];
			}
	        ret = usb_control_msg(usb_handle, 0x40, 0xe9, 0x0000, 0x0000, buf, bytesToXfer, 1000);
			if (ret!=bytesToXfer) {error=true;}
		}

	    // Perform SPI transfer
	    bytesToXfer = 3;
	    buf[0] = 0x06;
	    buf[1] = 0x02;
	    buf[2] = send_stop;
	    ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	    if (ret!=bytesToXfer) {error=true;}


	    // Read In buffer
		if (transfer_size > 64)
			bytesToXfer = 64;
		else
			bytesToXfer = transfer_size;
	    ret = usb_control_msg(usb_handle, 0xC0, 0xe8, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	    if (ret!=bytesToXfer) {error=true;}
	    for (i=0; i<bytesToXfer; i++) {
		    pInBuffer[i+bytes_sent] = buf[i];
	    }

		if (transfer_size > 64) {
			bytesToXfer = transfer_size - 64;
	        ret = usb_control_msg(usb_handle, 0xC0, 0xe9, 0x0000, 0x0000, buf, bytesToXfer, 1000);
			if (ret!=bytesToXfer) {error=true;}
			for (i=0; i<bytesToXfer; i++) {
				pInBuffer[i+64 + bytes_sent] = buf[i];
			}
		}

        // Update bytes sent
		bytes_sent = bytes_sent + transfer_size;

	}

	if (error==true) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}



NANO_RESULT	MBc::Nano_SPIRead ( 
		HANDLE 	hDevice, 
		BYTE 	Channel, 
		BYTE 	*pInBuffer, 
		WORD 	Length 
		)
{
    ret=-1;
	bool error;
	error = false;
    WORD bytes_left;
	WORD bytes_sent;
	BYTE transfer_size;
	BYTE send_stop;

	// Common Stuff
	int i;

    // Write to the CS channel (allows for GPIO based chip selects)
	bytesToXfer = 2;
	buf[0] = 0x02;
	buf[1] = Channel;
	ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	// Init SPI bus
	bytesToXfer = 2;
	buf[0] = 0x06;
	buf[1] = 0x01;
	ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

    bytes_left = Length;
    bytes_sent = 0;

	while (bytes_left>0) {
		
		if (bytes_left > 128) 
		{
			transfer_size = 128;
			send_stop = 0x0;
			bytes_left = bytes_left - 128;
		} else
		{
			transfer_size = bytes_left;
			send_stop = 0x1;
			bytes_left = 0;
		}

	    // Write to byte length register
	    bytesToXfer = 2;
	    buf[0] = 0x05;
	    buf[1] = transfer_size;
	    ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	    if (ret!=bytesToXfer) {error=true;}

	    // Perform SPI transfer
	    bytesToXfer = 3;
	    buf[0] = 0x06;
	    buf[1] = 0x03;
	    buf[2] = send_stop;
	    ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	    if (ret!=bytesToXfer) {error=true;}

	    // Read In buffer
		if (transfer_size > 64)
			bytesToXfer = 64;
		else
			bytesToXfer = transfer_size;
    	ret = usb_control_msg(usb_handle, 0xC0, 0xe8, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	    if (ret!=bytesToXfer) {error=true;}
	    for (i=0; i<bytesToXfer; i++) {
		    pInBuffer[i+bytes_sent] = buf[i];
	    }

		if (transfer_size > 64) {
			bytesToXfer = transfer_size - 64;
	        ret = usb_control_msg(usb_handle, 0xC0, 0xe9, 0x0000, 0x0000, buf, bytesToXfer, 1000);
			if (ret!=bytesToXfer) {error=true;}
			for (i=0; i<bytesToXfer; i++) {
				pInBuffer[i+64 + bytes_sent] = buf[i];
			}
		}

        // Update bytes sent
		bytes_sent = bytes_sent + transfer_size;

	}

	if (error==true) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}


NANO_RESULT	MBc::Nano_SPIWrite ( 
		HANDLE 	hDevice, 
		BYTE 	Channel, 
		BYTE 	*pOutBuffer, 
		WORD 	Length 
		)
{
    ret=-1;
	bool error;
	error = false;
    WORD bytes_left;
	WORD bytes_sent;
	BYTE transfer_size;
	BYTE send_stop;

	// Common Stuff
	int i;

    // Write to the CS channel (allows for GPIO based chip selects)
	bytesToXfer = 2;
	buf[0] = 0x02;
	buf[1] = Channel;
	ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

	// Init SPI bus
	bytesToXfer = 2;
	buf[0] = 0x06;
	buf[1] = 0x01;
	ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret!=bytesToXfer) {error=true;}

    bytes_left = Length;
    bytes_sent = 0;

	while (bytes_left>0) {
		
		if (bytes_left > 128) 
		{
			transfer_size = 128;
			send_stop = 0x0;
			bytes_left = bytes_left - 128;
		} else
		{
			transfer_size = bytes_left;
			send_stop = 0x1;
			bytes_left = 0;
		}

	    // Write to byte length register
	    bytesToXfer = 2;
	    buf[0] = 0x05;
	    buf[1] = transfer_size;
	    ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	    if (ret!=bytesToXfer) {error=true;}

	    // Init Out buffer
		if (transfer_size > 64)
			bytesToXfer = 64;
		else
			bytesToXfer = transfer_size;
	    for (i=0; i<bytesToXfer; i++) {
		    buf[i] = pOutBuffer[i+bytes_sent];
	    }
	    ret = usb_control_msg(usb_handle, 0x40, 0xe8, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	    if (ret!=bytesToXfer) {error=true;}

		if (transfer_size > 64) {
			bytesToXfer = transfer_size - 64;
			for (i=0; i<bytesToXfer; i++) {
				buf[i] = pOutBuffer[i+64 + bytes_sent];
			}
         	ret = usb_control_msg(usb_handle, 0x40, 0xe9, 0x0000, 0x0000, buf, bytesToXfer, 1000);
			if (ret!=bytesToXfer) {error=true;}
		}

	    // Perform SPI transfer
	    bytesToXfer = 3;
	    buf[0] = 0x06;
	    buf[1] = 0x04;
	    buf[2] = send_stop;
	    ret = usb_control_msg(usb_handle, 0x40, 0xe7, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	    if (ret!=bytesToXfer) {error=true;}


        // Update bytes sent
		bytes_sent = bytes_sent + transfer_size;

	}

	if (error==true) {return NANO_XACTION_FAILURE;} else {return NANO_SUCCESS;}
}



///////////////////////////////////////
// General Tasks
///////////////////////////////////////

bool MBc::OpenDevice() {

    /* Config USB trace debug, 0: None, 1-255: Trace depth */ 
    usb_set_debug( 0 );

    /* Initialize USB library */
    usb_init( );

    /* Make initial connection */
    usb_dev = device_init( VENDORID, PRODUCTID );
    if ( usb_dev == NULL )
    {
        usb_dev = device_init( 0x04B4, PRODUCTID );
        if ( usb_dev == NULL ) 
	{
		return false;
	}
	else
          // Open Device
          usb_handle = usb_open( usb_dev );
    }
	else
          // Open Device
          usb_handle = usb_open( usb_dev );
    if ( usb_handle == NULL )
    {
        return false;
    }
	return true;
}


WORD MBc::Nano_Revision() {
	// Get revision register
	WORD rslt;
	rslt = 0;
	bytesToXfer = 1;
    ret = usb_control_msg(usb_handle, 0xC0, 0xea, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret==bytesToXfer) {rslt = buf[0];}
    ret = usb_control_msg(usb_handle, 0xC0, 0xeb, 0x0000, 0x0000, buf, bytesToXfer, 1000);
	if (ret==bytesToXfer) {rslt=rslt<<8; rslt = rslt | buf[0];}
	return rslt;
}
