//////////////////////////////////////////////////////////////////////////
//
// Nano River Technologies
//
// File:         MiniBoard Class Library Header File (miniboard_class.h)
//
// Desciption:   This is the header file for the MiniBoard Class Library
//
// Revision:     Version 1.0
//
//////////////////////////////////////////////////////////////////////////
#pragma once

class MBc
{
public:
	MBc(void);
	virtual ~MBc(void);

	HANDLE mb_hDevice;


///////////////////////////////////////
// GPIO Tasks
///////////////////////////////////////

static	NANO_RESULT	Nano_GPIOSetDirection ( 
		HANDLE 	hDevice, 
		ULONG 	Value, 
		ULONG 	Mask
	);
	
static	NANO_RESULT	Nano_GPIOGetDirection (
		HANDLE 	hDevice, 
		ULONG  	*pValue 
	);

static	NANO_RESULT	Nano_GPIOWrite (
		HANDLE 	hDevice, 
		ULONG 	Value, 
		ULONG 	Mask
	);
	
static	NANO_RESULT	Nano_GPIORead (
		HANDLE 	hDevice, 
		ULONG 	*pValue
	);
	
static	NANO_RESULT	Nano_GPIOSetSingleBitDirection (
		HANDLE 	hDevice, 
		ULONG 	GPIONumber, 
		BOOL 	bOutput
	);
	
static	NANO_RESULT	Nano_GPIOGetSingleBitDirection (
		HANDLE 	hDevice, 
		ULONG 	GPIONumber, 
		BOOL 	*pbOutput
	);

static	NANO_RESULT	Nano_GPIOSingleBitWrite (
		HANDLE 	hDevice, 
		ULONG 	GPIONumber, 
		BOOL 	Value
	);
	
static	NANO_RESULT	Nano_GPIOSingleBitRead (
		HANDLE 	hDevice, 
		ULONG 	GPIONumber, 
		BOOL 	*pValue
	);



///////////////////////////////////////
// IIC Tasks
///////////////////////////////////////

static	NANO_RESULT	Nano_IICSetFrequency ( 
		HANDLE 	hDevice, 
		BYTE 	Frequency 
	);

static	NANO_RESULT	Nano_IICGetFrequency ( 
		HANDLE 	hDevice, 
		BYTE 	*pFrequency 
	);

static	NANO_RESULT	Nano_IICRead ( 
		HANDLE 	hDevice, 
		BYTE    SlaveAddress,
        BYTE    StartAddressLength,
		ULONG   StartAddress,
		WORD    BufferLength,
		BYTE    *pInBuffer
	);

static	NANO_RESULT	Nano_IICWrite ( 
		HANDLE 	hDevice, 
		BYTE    SlaveAddress,
        BYTE    StartAddressLength,
		ULONG   StartAddress,
		WORD    BufferLength,
		BYTE    *pOutBuffer
	);

static	NANO_RESULT	Nano_IICScanConnectedDevices ( 
		HANDLE 	hDevice, 
		DEV_LIST *pList
	);

static	NANO_RESULT	Nano_IICReadSCL ( 
		HANDLE 	hDevice, 
		BOOL 	*pState
	);

static	NANO_RESULT	Nano_IICReadSDA ( 
		HANDLE 	hDevice, 
		BOOL 	*pState
	);

static	NANO_RESULT	Nano_IICWriteSCL ( 
		HANDLE 	hDevice, 
		BOOL 	pState
	);

static	NANO_RESULT	Nano_IICWriteSDA ( 
		HANDLE 	hDevice, 
		BOOL 	pState
	);


///////////////////////////////////////
// SPI Tasks
///////////////////////////////////////

static	NANO_RESULT	Nano_SPISetClockConfig ( 
		HANDLE 	hDevice, 
		BYTE 	CPOL, 
		BYTE 	CPHA	
	);

static	NANO_RESULT	Nano_SPIGetClockConfig ( 
		HANDLE 	hDevice, 
		BYTE 	*pCPOL, 
		BYTE 	*pCPHA 
	);

static	NANO_RESULT	Nano_SPISetFrequency ( 
		HANDLE 	hDevice, 
		BYTE 	Frequency 
	);

static	NANO_RESULT	Nano_SPIGetFrequency ( 
		HANDLE 	hDevice, 
		BYTE 	*pFrequency		
	);

static	NANO_RESULT	Nano_SPISetChannelConfig ( 
		HANDLE 	hDevice, 
		BYTE 	Channel, 
		BYTE 	Configuration 
	);

static	NANO_RESULT	Nano_SPIGetChannelConfig ( 
		HANDLE 	hDevice, 
		BYTE 	Channel, 
		BYTE 	*pConfiguration 
	);

static	NANO_RESULT	Nano_SPIReadWrite ( 
		HANDLE 	hDevice, 
		BYTE 	Channel, 
		BYTE 	*pInBuffer, 
		BYTE 	*pOutBuffer, 
		WORD 	Length
	);

static	NANO_RESULT	Nano_SPIRead ( 
		HANDLE 	hDevice, 
		BYTE 	Channel, 
		BYTE 	*pInBuffer, 
		WORD 	Length 
	);

static	NANO_RESULT	Nano_SPIWrite ( 
		HANDLE 	hDevice, 
		BYTE 	Channel, 
		BYTE 	*pOutBuffer, 
		WORD 	Length 
	);




///////////////////////////////////////
// General Tasks
///////////////////////////////////////

static bool OpenDevice();

static WORD Nano_Revision();


};


